/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/glint.h>

#include <iostream>

using namespace glint;
using namespace std;

const string v_shader = "attribute vec4 pos;"
                        "void main()"
                        "{"
                        "	gl_Position = pos;"
                        "}";

const string f_shader = "uniform vec4 color;"
                        "void main()"
                        "{"
	                    "    gl_FragColor = color;"
                        "}";

int
main()
{
    try
    {
        auto rt = Runtime::create();
        rt->create_display(300, 200);

        Program::UPtr prog;
        {
            auto vs = VertexShader::create(v_shader);
            auto fs = FragmentShader::create(f_shader);
            prog = Program::create(*vs, *fs);
        }
        prog->use();

        prog->uniform("color") = FloatVec4({ { 0.5f, 0.5f, 0.5f, 1.0f } });
        FloatVec3Array v = { {
                                { {  0.0f,  0.5f,  0.0f } },
                                { { -0.5f, -0.5f,  0.0f } },
                                { {  0.5f, -0.5f,  0.0f } }
                           } };

        glBindAttribLocation(prog->id(), 0, "pos");
        glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, &v[0][0]);
        glEnableVertexAttribArray(0);

        glDrawArrays(GL_TRIANGLES, 0, v.size());
        check_gl("glDrawArrays() failed");

        rt->display()->swap_buffers();
        sleep(3);
    }
    catch (const Exception& e)
    {
        cerr << e.to_string() << endl;
        exit(1);
    }
}
