/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include "cone.h"

#include <cassert>
#include <cmath>
#include <iostream> // TODO: remove this
#include <iomanip> // TODO: remove this

using namespace glint;
using namespace std;

void
create_cone(int segments, FloatVec3Array& vertices, FloatVec3Array& normals)
{
    assert(segments > 2);

    vertices.resize(segments + 2);  // One extra element for the tip and one to close the cone
    normals.resize(segments + 2);

    // The cone is centered on origin, with the base vertices on a unit circle parallel to the xz plane.
    // The base is at -1 below the plane, and the tip is at +1 above the plane.
    // The tip is the first element on the vector and becomes the anchor of a triangle fan.

    FloatVec3 tip = { { 0, 1, 0 } };
    vertices[0] = tip;
    normals[0] = tip;

    constexpr float normal_y = atan(1.0f / 2);  // Angle of the side of the cone

    FloatVec3 p;
    for (int i = 1; i < segments + 1; ++i)
    {
        float radians = 2.0f * M_PI * (i - 1) / segments;
        p[0] = cos(radians);
        p[1] = 0;
        p[2] = -sin(radians);
        vertices[i] = p;        // Base vertex
        p[1] = normal_y;
        normals[i] = p;         // Base normal
    }

    // Last base vertex is the same as the first base vertex, which closes the cone.
    vertices[segments + 1] = vertices[1];
    normals[segments + 1] = vertices[1];

    for (int i = 1; i <= segments + 1; ++i)
    {
        cout << setprecision(6) << "x: " << setw(12) << vertices[i][0] << ", z: " << setw(12) << vertices[i][2] << endl;
    }

}
