/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_INTERNAL_ATTRIBUTEVARIABLEIMPL_H
#define GLINT_INTERNAL_ATTRIBUTEVARIABLEIMPL_H

#include <glint/AttributeVariable.h>
#include <glint/internal/ShaderVariableImpl.h>
#include <glint/Types.h>

namespace glint
{

namespace internal
{

class AttributeVariableImpl : public ShaderVariableImpl
{
public:
    AttributeVariable::ArrayState array_state() const;
    void set_array_state(AttributeVariable::ArrayState state);

protected:
    AttributeVariableImpl(GLuint prog_id, GLint location, const std::string& name, GLenum type);
    virtual ~AttributeVariableImpl() noexcept;
};

class FloatAttributeImpl : public AttributeVariableImpl
{
public:
    FloatAttributeImpl(GLuint prog_id, GLint location, const std::string& name);
    ~FloatAttributeImpl() noexcept;

    GLfloat get() const;
    void set(GLfloat v);
};

class FloatVec2AttributeImpl : public AttributeVariableImpl
{
public:
    FloatVec2AttributeImpl(GLuint prog_id, GLint location, const std::string& name);
    virtual ~FloatVec2AttributeImpl() noexcept;

    FloatVec2 get() const;
    void set(const FloatVec2& v);
};

class FloatVec3AttributeImpl : public AttributeVariableImpl
{
public:
    FloatVec3AttributeImpl(GLuint prog_id, GLint location, const std::string& name);
    virtual ~FloatVec3AttributeImpl() noexcept;

    FloatVec3 get() const;
    void set(const FloatVec3& v);
};

class FloatVec4AttributeImpl : public AttributeVariableImpl
{
public:
    FloatVec4AttributeImpl(GLuint prog_id, GLint location, const std::string& name);
    virtual ~FloatVec4AttributeImpl() noexcept;

    FloatVec4 get() const;
    void set(const FloatVec4& v);
};

class FloatMat2AttributeImpl : public AttributeVariableImpl
{
public:
    FloatMat2AttributeImpl(GLuint prog_id, GLint location, const std::string& name);
    virtual ~FloatMat2AttributeImpl() noexcept;

    FloatMat2 get() const;
    void set(const FloatMat2& v);
};

class FloatMat3AttributeImpl : public AttributeVariableImpl
{
public:
    FloatMat3AttributeImpl(GLuint prog_id, GLint location, const std::string& name);
    virtual ~FloatMat3AttributeImpl() noexcept;

    FloatMat3 get() const;
    void set(const FloatMat3& v);
};

class FloatMat4AttributeImpl : public AttributeVariableImpl
{
public:
    FloatMat4AttributeImpl(GLuint prog_id, GLint location, const std::string& name);
    virtual ~FloatMat4AttributeImpl() noexcept;

    FloatMat4 get() const;
    void set(const FloatMat4& v);
};

} // namespace internal

} // namespace glint

#endif
