/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/AttributeVariable.h>
#include <glint/internal/AttributeVariableImpl.h>

using namespace glint;
using namespace std;

namespace glint
{

// AttributeVariable

AttributeVariable::
AttributeVariable(const shared_ptr<internal::AttributeVariableImpl>& derived)
    : ShaderVariable(derived)
{
}

AttributeVariable::
~AttributeVariable() = default;


AttributeVariable::ArrayState
AttributeVariable::
array_state() const
{
    return dynamic_pointer_cast<internal::AttributeVariableImpl>(pimpl())->array_state();
}

void
AttributeVariable::
set_array_state(ArrayState state)
{
    dynamic_pointer_cast<internal::AttributeVariableImpl>(pimpl())->set_array_state(state);
}

void
AttributeVariable::
set(GLfloat rhs)
{
    FloatAttribute& u = dynamic_cast<FloatAttribute&>(*this);
    u.set(rhs);
}

AttributeVariable&
AttributeVariable::
operator=(GLfloat rhs)
{
    FloatAttribute& u = dynamic_cast<FloatAttribute&>(*this);
    u.set(rhs);
    return u;
}

void
AttributeVariable::
set(const FloatVec2& rhs)
{
    FloatVec2Attribute& u = dynamic_cast<FloatVec2Attribute&>(*this);
    u.set(rhs);
}

AttributeVariable&
AttributeVariable::
operator=(const FloatVec2& rhs)
{
    FloatVec2Attribute& u = dynamic_cast<FloatVec2Attribute&>(*this);
    u.set(rhs);
    return u;
}

void
AttributeVariable::
set(const FloatVec3& rhs)
{
    FloatVec3Attribute& u = dynamic_cast<FloatVec3Attribute&>(*this);
    u.set(rhs);
}

AttributeVariable&
AttributeVariable::
operator=(const FloatVec3& rhs)
{
    FloatVec3Attribute& u = dynamic_cast<FloatVec3Attribute&>(*this);
    u.set(rhs);
    return u;
}

void
AttributeVariable::
set(const FloatVec4& rhs)
{
    FloatVec4Attribute& u = dynamic_cast<FloatVec4Attribute&>(*this);
    u.set(rhs);
}

AttributeVariable&
AttributeVariable::
operator=(const FloatVec4& rhs)
{
    FloatVec4Attribute& u = dynamic_cast<FloatVec4Attribute&>(*this);
    u.set(rhs);
    return u;
}

void
AttributeVariable::
set(const FloatMat2& rhs)
{
    FloatMat2Attribute& u = dynamic_cast<FloatMat2Attribute&>(*this);
    u.set(rhs);
}

AttributeVariable&
AttributeVariable::
operator=(const FloatMat2& rhs)
{
    FloatMat2Attribute& u = dynamic_cast<FloatMat2Attribute&>(*this);
    u.set(rhs);
    return u;
}

void
AttributeVariable::
set(const FloatMat3& rhs)
{
    FloatMat3Attribute& u = dynamic_cast<FloatMat3Attribute&>(*this);
    u.set(rhs);
}

AttributeVariable&
AttributeVariable::
operator=(const FloatMat3& rhs)
{
    FloatMat3Attribute& u = dynamic_cast<FloatMat3Attribute&>(*this);
    u.set(rhs);
    return u;
}

void
AttributeVariable::
set(const FloatMat4& rhs)
{
    FloatMat4Attribute& u = dynamic_cast<FloatMat4Attribute&>(*this);
    u.set(rhs);
}

AttributeVariable&
AttributeVariable::
operator=(const FloatMat4& rhs)
{
    FloatMat4Attribute& u = dynamic_cast<FloatMat4Attribute&>(*this);
    u.set(rhs);
    return u;
}

// Float

FloatAttribute::
FloatAttribute(GLuint prog_id, GLint location, const std::string& name)
    : AttributeVariable(make_shared<internal::FloatAttributeImpl>(prog_id, location, name))
{
}

FloatAttribute::
~FloatAttribute() = default;

GLfloat
FloatAttribute::
get() const
{
    return dynamic_pointer_cast<internal::FloatAttributeImpl>(pimpl())->get();
}

void
FloatAttribute::
set(GLfloat v)
{
    dynamic_pointer_cast<internal::FloatAttributeImpl>(pimpl())->set(v);
}

FloatAttribute::
operator GLfloat() const
{
    return get();
}

FloatAttribute&
FloatAttribute::
operator=(GLfloat rhs)
{
    set(rhs);
    return *this;
}

FloatAttribute&
FloatAttribute::
operator=(const FloatAttribute& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatVec2

FloatVec2Attribute::
FloatVec2Attribute(GLuint prog_id, GLint location, const string& name)
    : AttributeVariable(make_shared<internal::FloatVec2AttributeImpl>(prog_id, location, name))
{
}

FloatVec2Attribute::
~FloatVec2Attribute() = default;

FloatVec2
FloatVec2Attribute::
get() const
{
    return dynamic_pointer_cast<internal::FloatVec2AttributeImpl>(pimpl())->get();
}

void
FloatVec2Attribute::
get(GLfloat& x, GLfloat& y) const
{
    FloatVec2 v = dynamic_pointer_cast<internal::FloatVec2AttributeImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
}

void
FloatVec2Attribute::
set(const FloatVec2& v)
{
    dynamic_pointer_cast<internal::FloatVec2AttributeImpl>(pimpl())->set(v);
}

void
FloatVec2Attribute::
set(GLfloat x, GLfloat y)
{
    FloatVec2 v = { { x, y } };
    dynamic_pointer_cast<internal::FloatVec2AttributeImpl>(pimpl())->set(v);
}

FloatVec2Attribute::
operator FloatVec2() const
{
    return get();
}

FloatVec2Attribute&
FloatVec2Attribute::
operator=(const FloatVec2& rhs)
{
    set(rhs);
    return *this;
}

FloatVec2Attribute&
FloatVec2Attribute::
operator=(const FloatVec2Attribute& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatVec3

FloatVec3Attribute::
FloatVec3Attribute(GLuint prog_id, GLint location, const string& name)
    : AttributeVariable(make_shared<internal::FloatVec3AttributeImpl>(prog_id, location, name))
{
}

FloatVec3Attribute::
~FloatVec3Attribute() = default;

FloatVec3
FloatVec3Attribute::
get() const
{
    return dynamic_pointer_cast<internal::FloatVec3AttributeImpl>(pimpl())->get();
}

void
FloatVec3Attribute::
get(GLfloat& x, GLfloat& y, GLfloat& z) const
{
    FloatVec3 v = dynamic_pointer_cast<internal::FloatVec3AttributeImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
    z = v[2];
}

void
FloatVec3Attribute::
set(const FloatVec3& v)
{
    dynamic_pointer_cast<internal::FloatVec3AttributeImpl>(pimpl())->set(v);
}

void
FloatVec3Attribute::
set(GLfloat x, GLfloat y, GLfloat z)
{
    FloatVec3 v = { { x, y, z } };
    dynamic_pointer_cast<internal::FloatVec3AttributeImpl>(pimpl())->set(v);
}

FloatVec3Attribute::
operator FloatVec3() const
{
    return get();
}

FloatVec3Attribute&
FloatVec3Attribute::
operator=(const FloatVec3& rhs)
{
    set(rhs);
    return *this;
}

FloatVec3Attribute&
FloatVec3Attribute::
operator=(const FloatVec3Attribute& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatVec4

FloatVec4Attribute::
FloatVec4Attribute(GLuint prog_id, GLint location, const string& name)
    : AttributeVariable(make_shared<internal::FloatVec4AttributeImpl>(prog_id, location, name))
{
}

FloatVec4Attribute::
~FloatVec4Attribute() = default;

FloatVec4
FloatVec4Attribute::
get() const
{
    return dynamic_pointer_cast<internal::FloatVec4AttributeImpl>(pimpl())->get();
}

void
FloatVec4Attribute::
get(GLfloat& x, GLfloat& y, GLfloat& z, GLfloat& w) const
{
    FloatVec4 v = dynamic_pointer_cast<internal::FloatVec4AttributeImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
    z = v[2];
    w = v[3];
}

void
FloatVec4Attribute::
set(const FloatVec4& v)
{
    dynamic_pointer_cast<internal::FloatVec4AttributeImpl>(pimpl())->set(v);
}

void
FloatVec4Attribute::
set(GLfloat x, GLfloat y, GLfloat z, GLfloat w)
{
    FloatVec4 v = { { x, y, z, w } };
    dynamic_pointer_cast<internal::FloatVec4AttributeImpl>(pimpl())->set(v);
}

FloatVec4Attribute::
operator FloatVec4() const
{
    return get();
}

FloatVec4Attribute&
FloatVec4Attribute::
operator=(const FloatVec4& rhs)
{
    set(rhs);
    return *this;
}

FloatVec4Attribute&
FloatVec4Attribute::
operator=(const FloatVec4Attribute& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatMat2

FloatMat2Attribute::
FloatMat2Attribute(GLuint prog_id, GLint location, const string& name)
    : AttributeVariable(make_shared<internal::FloatMat2AttributeImpl>(prog_id, location, name))
{
}

FloatMat2Attribute::
~FloatMat2Attribute() = default;

FloatMat2
FloatMat2Attribute::
get() const
{
    return dynamic_pointer_cast<internal::FloatMat2AttributeImpl>(pimpl())->get();
}

void
FloatMat2Attribute::
set(const FloatMat2& v)
{
    dynamic_pointer_cast<internal::FloatMat2AttributeImpl>(pimpl())->set(v);
}

FloatMat2Attribute::
operator FloatMat2() const
{
    return get();
}

FloatMat2Attribute&
FloatMat2Attribute::
operator=(const FloatMat2& rhs)
{
    set(rhs);
    return *this;
}

FloatMat2Attribute&
FloatMat2Attribute::
operator=(const FloatMat2Attribute& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatMat3

FloatMat3Attribute::
FloatMat3Attribute(GLuint prog_id, GLint location, const string& name)
    : AttributeVariable(make_shared<internal::FloatMat3AttributeImpl>(prog_id, location, name))
{
}

FloatMat3Attribute::
~FloatMat3Attribute() = default;

FloatMat3
FloatMat3Attribute::
get() const
{
    return dynamic_pointer_cast<internal::FloatMat3AttributeImpl>(pimpl())->get();
}

void
FloatMat3Attribute::
set(const FloatMat3& v)
{
    dynamic_pointer_cast<internal::FloatMat3AttributeImpl>(pimpl())->set(v);
}

FloatMat3Attribute::
operator FloatMat3() const
{
    return get();
}

FloatMat3Attribute&
FloatMat3Attribute::
operator=(const FloatMat3& rhs)
{
    set(rhs);
    return *this;
}

FloatMat3Attribute&
FloatMat3Attribute::
operator=(const FloatMat3Attribute& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatMat4

FloatMat4Attribute::
FloatMat4Attribute(GLuint prog_id, GLint location, const string& name)
    : AttributeVariable(make_shared<internal::FloatMat4AttributeImpl>(prog_id, location, name))
{
}

FloatMat4Attribute::
~FloatMat4Attribute() = default;

FloatMat4
FloatMat4Attribute::
get() const
{
    return dynamic_pointer_cast<internal::FloatMat4AttributeImpl>(pimpl())->get();
}

void
FloatMat4Attribute::
set(const FloatMat4& v)
{
    dynamic_pointer_cast<internal::FloatMat4AttributeImpl>(pimpl())->set(v);
}

FloatMat4Attribute::
operator FloatMat4() const
{
    return get();
}

FloatMat4Attribute&
FloatMat4Attribute::
operator=(const FloatMat4& rhs)
{
    set(rhs);
    return *this;
}

FloatMat4Attribute&
FloatMat4Attribute::
operator=(const FloatMat4Attribute& rhs)
{
    set(rhs.get());
    return *this;
}

} // namespace glint
